import type { SceneController, SceneType } from "../base"
import type { InteractionSceneDefinition } from "../../../shared/script/types"
import PlayerView from "./PlayerView.vue"
import type { InteractionSceneEvent } from "../../../shared/scene-types/interaction"
import DuoView from "./DuoView.vue"
import type { SuggestedInteraction } from "../../../shared/mutations"
import { computed, reactive, type Reactive, type Ref } from "vue"
import { trpcClient } from "../../trpc"

export const InteractionSceneType: SceneType<InteractionSceneDefinition, InteractionSceneController> = {
    id: "interaction",
    playerView: PlayerView,
    duoView: DuoView,
    createController(definition: InteractionSceneDefinition): InteractionSceneController {
        const visibleObjectIds = reactive(new Set<string>())
        const interactionQueue = reactive(new Map<string, InteractionQueueItem>())

        return {
            visibleObjectIds,
            interactionQueue,
            sortedInteractionQueue: computed(() => [...interactionQueue.values()].sort((a, b) => a.votes - b.votes)),
            handleEvent(event: InteractionSceneEvent) {

            },
            async setObjectVisibility(objectId: string, isVisible: boolean) {
                await trpcClient.crew.interactionScene.setObjectVisibility.mutate({ objectId, isVisible })
            }
        }
    }
}

export interface InteractionQueueItem {
    id: string
    interaction: SuggestedInteraction
    votes: number
}

export interface InteractionSceneController extends SceneController {
    visibleObjectIds: Reactive<Set<string>>
    interactionQueue: Reactive<Map<string, InteractionQueueItem>>
    sortedInteractionQueue: Readonly<Ref<Array<InteractionQueueItem>>>

    setObjectVisibility(objectId: string, isVisible: boolean): Promise<void>
}